@@ -1,7 +1,9 @@
+import copy
 from functools import partial
-from math import isnan
+from math import isnan, sqrt
 
 import dask.array as da
+import numba as nb
 import numpy as np
 import pandas as pd
 import xarray as xr
@@ -98,7 +100,7 @@ def _mean_cupy(data, excludes):
 
 
 def _mean(data, excludes):
-    agg = xr.DataArray(data.astype(float))
+    agg = xr.DataArray(data)
     mapper = ArrayTypeFunctionMapping(
         numpy_func=_mean_numpy,
         cupy_func=_mean_cupy,
@@ -191,15 +193,15 @@ def mean(agg, passes=1, excludes=[np.nan], name='mean'):
         <class 'cupy.core.core.ndarray'>
         >>> print(mean_cupy)
         <xarray.DataArray 'mean' (dim_0: 5, dim_1: 5)>
-        array([[0.47928994, 0.47928994, 0.47928994, 0.47928994, 0.47928994],
-               [0.47928994, 0.47928994, 0.47928994, 0.47928994, 0.47928994],
-               [0.47928994, 0.47928994, 0.47928994, 0.47928994, 0.47928994],
-               [0.47928994, 0.47928994, 0.47928994, 0.47928994, 0.47928994],
-               [0.47928994, 0.47928994, 0.47928994, 0.47928994, 0.47928994]])
+        array([[0.47928995, 0.47928995, 0.47928995, 0.47928995, 0.47928995],
+               [0.47928995, 0.47928995, 0.47928995, 0.47928995, 0.47928995],
+               [0.47928995, 0.47928995, 0.47928995, 0.47928995, 0.47928995],
+               [0.47928995, 0.47928995, 0.47928995, 0.47928995, 0.47928995],
+               [0.47928995, 0.47928995, 0.47928995, 0.47928995, 0.47928995]])
         Dimensions without coordinates: dim_0, dim_1
     """
 
-    out = agg.data
+    out = agg.data.astype(float)
     for i in range(passes):
         out = _mean(out, tuple(excludes))
 
@@ -285,33 +287,6 @@ def _apply_dask_numpy(data, kernel, func):
     return out
 
 
-def _apply_cupy(data, kernel, func):
-    data = data.astype(cupy.float32)
-    kernel = cupy.asarray(kernel)
-
-    out = cupy.zeros(data.shape, dtype=data.dtype)
-    out[:] = cupy.nan
-
-    rows, cols = data.shape
-    krows, kcols = kernel.shape
-    hrows, hcols = int(krows / 2), int(kcols / 2)
-    kernel_values = cupy.zeros_like(kernel, dtype=data.dtype)
-
-    for y in prange(rows):
-        for x in prange(cols):
-            # kernel values are all nans at the beginning of each step
-            kernel_values.fill(np.nan)
-            for ky in range(y - hrows, y + hrows + 1):
-                for kx in range(x - hcols, x + hcols + 1):
-                    if ky >= 0 and ky < rows and kx >= 0 and kx < cols:
-                        kyidx, kxidx = ky - (y - hrows), kx - (x - hcols)
-                        if kernel[kyidx, kxidx] == 1:
-                            kernel_values[kyidx, kxidx] = data[ky, kx]
-            out[y, x] = func(kernel_values)
-
-    return out
-
-
 def apply(raster, kernel, func=_calc_mean, name='focal_apply'):
     """
     Returns custom function applied array using a user-created window.
@@ -320,7 +295,7 @@ def apply(raster, kernel, func=_calc_mean, name='focal_apply'):
     ----------
     raster : xarray.DataArray
         2D array of input values to be filtered. Can be a NumPy backed,
-        CuPy backed, or Dask with NumPy backed DataArray.
+        or Dask with NumPy backed DataArray.
     kernel : numpy.ndarray
         2D array where values of 1 indicate the kernel.
     func : callable, default=xrspatial.focal._calc_mean
@@ -356,6 +331,7 @@ def apply(raster, kernel, func=_calc_mean, name='focal_apply'):
                [0., 1., 0.]])
         >>> # apply kernel mean by default
         >>> apply_mean_agg = apply(raster, kernel)
+        >>> apply_mean_agg
         <xarray.DataArray 'focal_apply' (y: 4, x: 5)>
         array([[ 2.        ,  2.25   ,  3.25      ,  4.25      ,  5.33333333],
                [ 5.25      ,  6.     ,  7.        ,  8.        ,  8.75      ],
@@ -385,13 +361,14 @@ def apply(raster, kernel, func=_calc_mean, name='focal_apply'):
     ])
     >>> @ngjit
     >>> def func(kernel_data):
-    >>>     weight = np.array([
-                [0, 0.5, 0],
-                [0, 1, 0.5],
-                [0, 0.5, 0],
-            ])
-    >>>    return np.nansum(kernel_data * weight)
+    ...     weight = np.array([
+    ...         [0, 0.5, 0],
+    ...         [0, 1, 0.5],
+    ...         [0, 0.5, 0],
+    ...     ])
+    ...     return np.nansum(kernel_data * weight)
 
+    >>> import dask.array as da
     >>> data_da = da.from_array(np.ones((6, 4), dtype=np.float64), chunks=(3, 2))
     >>> raster_da = xr.DataArray(data_da, dims=['y', 'x'], name='raster_da')
     >>> print(raster_da)
@@ -428,10 +405,11 @@ def apply(raster, kernel, func=_calc_mean, name='focal_apply'):
     # the function func must be a @ngjit
     mapper = ArrayTypeFunctionMapping(
         numpy_func=_apply_numpy,
-        cupy_func=_apply_cupy,
+        cupy_func=lambda *args: not_implemented_func(
+            *args, messages='apply() does not support cupy backed DataArray.'),
         dask_func=_apply_dask_numpy,
         dask_cupy_func=lambda *args: not_implemented_func(
-            *args, messages='apply() does not support dask with cupy backed DataArray.'),  # noqa
+            *args, messages='apply() does not support dask with cupy backed DataArray.'),
     )
     out = mapper(raster)(raster.data, kernel, func)
     result = DataArray(out,
@@ -442,6 +420,178 @@ def apply(raster, kernel, func=_calc_mean, name='focal_apply'):
     return result
 
 
+@cuda.jit
+def _focal_min_cuda(data, kernel, out):
+    i, j = cuda.grid(2)
+
+    delta_rows = kernel.shape[0] // 2
+    delta_cols = kernel.shape[1] // 2
+
+    data_rows, data_cols = data.shape
+
+    if i < delta_rows or i >= data_rows - delta_rows or \
+            j < delta_cols or j >= data_cols - delta_cols:
+        return
+
+    s = data[i, j]
+    for k in range(kernel.shape[0]):
+        for h in range(kernel.shape[1]):
+            i_k = i + k - delta_rows
+            j_h = j + h - delta_cols
+            if (i_k >= 0) and (i_k < data_rows) and (j_h >= 0) and (j_h < data_cols):
+                if (kernel[k, h] != 0) and s > data[i_k, j_h]:
+                    s = data[i_k, j_h]
+    out[i, j] = s
+
+
+@cuda.jit
+def _focal_max_cuda(data, kernel, out):
+    i, j = cuda.grid(2)
+
+    delta_rows = kernel.shape[0] // 2
+    delta_cols = kernel.shape[1] // 2
+
+    data_rows, data_cols = data.shape
+
+    if i < delta_rows or i >= data_rows - delta_rows or \
+            j < delta_cols or j >= data_cols - delta_cols:
+        return
+
+    s = data[i, j]
+    for k in range(kernel.shape[0]):
+        for h in range(kernel.shape[1]):
+            i_k = i + k - delta_rows
+            j_h = j + h - delta_cols
+            if (i_k >= 0) and (i_k < data_rows) and (j_h >= 0) and (j_h < data_cols):
+                if (kernel[k, h] != 0) and s < data[i_k, j_h]:
+                    s = data[i_k, j_h]
+    out[i, j] = s
+
+
+def _focal_range_cupy(data, kernel):
+    focal_min = _focal_stats_func_cupy(data, kernel, _focal_min_cuda)
+    focal_max = _focal_stats_func_cupy(data, kernel, _focal_max_cuda)
+    out = focal_max - focal_min
+    return out
+
+
+@cuda.jit
+def _focal_std_cuda(data, kernel, out):
+    i, j = cuda.grid(2)
+
+    delta_rows = kernel.shape[0] // 2
+    delta_cols = kernel.shape[1] // 2
+
+    data_rows, data_cols = data.shape
+
+    if i < delta_rows or i >= data_rows - delta_rows or \
+            j < delta_cols or j >= data_cols - delta_cols:
+        return
+
+    sum_squares = 0
+    sum = 0
+    count = 0
+    for k in range(kernel.shape[0]):
+        for h in range(kernel.shape[1]):
+            i_k = i + k - delta_rows
+            j_h = j + h - delta_cols
+            if (i_k >= 0) and (i_k < data_rows) and (j_h >= 0) and (j_h < data_cols):
+                sum_squares += (kernel[k, h]*data[i_k, j_h])**2
+                sum += kernel[k, h]*data[i_k, j_h]
+                count += kernel[k, h]
+    squared_sum = sum**2
+    out[i, j] = sqrt((sum_squares - squared_sum/count) / count)
+
+
+@cuda.jit
+def _focal_var_cuda(data, kernel, out):
+    i, j = cuda.grid(2)
+
+    delta_rows = kernel.shape[0] // 2
+    delta_cols = kernel.shape[1] // 2
+
+    data_rows, data_cols = data.shape
+
+    if i < delta_rows or i >= data_rows - delta_rows or \
+            j < delta_cols or j >= data_cols - delta_cols:
+        return
+
+    sum_squares = 0
+    sum = 0
+    count = 0
+    for k in range(kernel.shape[0]):
+        for h in range(kernel.shape[1]):
+            i_k = i + k - delta_rows
+            j_h = j + h - delta_cols
+            if (i_k >= 0) and (i_k < data_rows) and (j_h >= 0) and (j_h < data_cols):
+                sum_squares += (kernel[k, h]*data[i_k, j_h])**2
+                sum += kernel[k, h]*data[i_k, j_h]
+                count += kernel[k, h]
+    squared_sum = sum**2
+    out[i, j] = (sum_squares - squared_sum/count) / count
+
+
+def _focal_mean_cupy(data, kernel):
+    out = convolve_2d(data, kernel / kernel.sum())
+    return out
+
+
+def _focal_sum_cupy(data, kernel):
+    out = convolve_2d(data, kernel)
+    return out
+
+
+def _focal_stats_func_cupy(data, kernel, func=_focal_max_cuda):
+    out = cupy.empty(data.shape, dtype='f4')
+    out[:, :] = cupy.nan
+    griddim, blockdim = cuda_args(data.shape)
+    func[griddim, blockdim](data, kernel, cupy.asarray(out))
+    return out
+
+
+def _focal_stats_cupy(agg, kernel, stats_funcs):
+    _stats_cupy_mapper = dict(
+        mean=_focal_mean_cupy,
+        sum=_focal_sum_cupy,
+        range=_focal_range_cupy,
+        max=lambda *args: _focal_stats_func_cupy(*args, func=_focal_max_cuda),
+        min=lambda *args: _focal_stats_func_cupy(*args, func=_focal_min_cuda),
+        std=lambda *args: _focal_stats_func_cupy(*args, func=_focal_std_cuda),
+        var=lambda *args: _focal_stats_func_cupy(*args, func=_focal_var_cuda),
+    )
+    stats_aggs = []
+    for stats in stats_funcs:
+        data = agg.data.astype(cupy.float32)
+        stats_data = _stats_cupy_mapper[stats](data, kernel)
+        stats_agg = xr.DataArray(
+            stats_data,
+            dims=agg.dims,
+            coords=agg.coords,
+            attrs=agg.attrs
+          )
+        stats_aggs.append(stats_agg)
+    stats = xr.concat(stats_aggs, pd.Index(stats_funcs, name='stats'))
+    return stats
+
+
+def _focal_stats_cpu(agg, kernel, stats_funcs):
+    _function_mapping = {
+        'mean': _calc_mean,
+        'max': _calc_max,
+        'min': _calc_min,
+        'range': _calc_range,
+        'std': _calc_std,
+        'var': _calc_var,
+        'sum': _calc_sum
+    }
+    stats_aggs = []
+    for stats in stats_funcs:
+        stats_agg = apply(agg, kernel, func=_function_mapping[stats])
+        stats_aggs.append(stats_agg)
+    stats = xr.concat(stats_aggs, pd.Index(stats_funcs, name='stats'))
+    return stats
+
+
 def focal_stats(agg,
                 kernel,
                 stats_funcs=[
@@ -502,24 +652,25 @@ def focal_stats(agg,
           * stats    (stats) object 'min' 'sum'
         Dimensions without coordinates: dim_0, dim_1
     """
+    # validate raster
+    if not isinstance(agg, DataArray):
+        raise TypeError("`agg` must be instance of DataArray")
 
-    _function_mapping = {
-        'mean': _calc_mean,
-        'max': _calc_max,
-        'min': _calc_min,
-        'range': _calc_range,
-        'std': _calc_std,
-        'var': _calc_var,
-        'sum': _calc_sum
-    }
+    if agg.ndim != 2:
+        raise ValueError("`agg` must be 2D")
 
-    stats_aggs = []
-    for stats in stats_funcs:
-        stats_agg = apply(agg, kernel, func=_function_mapping[stats])
-        stats_aggs.append(stats_agg)
+    # Validate the kernel
+    kernel = custom_kernel(kernel)
 
-    stats = xr.concat(stats_aggs, pd.Index(stats_funcs, name='stats'))
-    return stats
+    mapper = ArrayTypeFunctionMapping(
+        numpy_func=_focal_stats_cpu,
+        cupy_func=_focal_stats_cupy,
+        dask_func=_focal_stats_cpu,
+        dask_cupy_func=lambda *args: not_implemented_func(
+            *args, messages='focal_stats() does not support dask with cupy backed DataArray.'),
+    )
+    result = mapper(agg)(agg, kernel, stats_funcs)
+    return result
 
 
 @ngjit
@@ -610,40 +761,42 @@ def _hotspots_dask_numpy(raster, kernel):
     return out
 
 
-def _calc_hotspots_cupy(z_array):
-    out = cupy.zeros_like(z_array, dtype=cupy.int8)
-    rows, cols = z_array.shape
+@nb.cuda.jit(device=True)
+def _gpu_hotspots(data):
+    zscore = data[0, 0]
 
-    for y in prange(rows):
-        for x in prange(cols):
-            zscore = z_array[y, x]
+    # find p value
+    p_value = 1.0
+    if abs(zscore) >= 2.33:
+        p_value = 0.0099
+    elif abs(zscore) >= 1.65:
+        p_value = 0.0495
+    elif abs(zscore) >= 1.29:
+        p_value = 0.0985
 
-            # find p value
-            p_value = 1.0
-            if abs(zscore) >= 2.33:
-                p_value = 0.0099
-            elif abs(zscore) >= 1.65:
-                p_value = 0.0495
-            elif abs(zscore) >= 1.29:
-                p_value = 0.0985
+    # confidence
+    confidence = 0
+    if abs(zscore) > 2.58 and p_value < 0.01:
+        confidence = 99
+    elif abs(zscore) > 1.96 and p_value < 0.05:
+        confidence = 95
+    elif abs(zscore) > 1.65 and p_value < 0.1:
+        confidence = 90
 
-            # confidence
-            confidence = 0
-            if abs(zscore) > 2.58 and p_value < 0.01:
-                confidence = 99
-            elif abs(zscore) > 1.96 and p_value < 0.05:
-                confidence = 95
-            elif abs(zscore) > 1.65 and p_value < 0.1:
-                confidence = 90
+    hot_cold = 0
+    if zscore > 0:
+        hot_cold = 1
+    elif zscore < 0:
+        hot_cold = -1
 
-            hot_cold = 0
-            if zscore > 0:
-                hot_cold = 1
-            elif zscore < 0:
-                hot_cold = -1
+    return hot_cold * confidence
 
-            out[y, x] = hot_cold * confidence
-    return out
+
+@nb.cuda.jit
+def _run_gpu_hotspots(data, out):
+    i, j = nb.cuda.grid(2)
+    if i >= 0 and i < out.shape[0] and j >= 0 and j < out.shape[1]:
+        out[i, j] = _gpu_hotspots(data[i:i + 1, j:j + 1])
 
 
 def _hotspots_cupy(raster, kernel):
@@ -665,7 +818,9 @@ def _hotspots_cupy(raster, kernel):
         )
     z_array = (mean_array - global_mean) / global_std
 
-    out = _calc_hotspots_cupy(z_array)
+    out = cupy.zeros_like(z_array, dtype=cupy.int8)
+    griddim, blockdim = cuda_args(z_array.shape)
+    _run_gpu_hotspots[griddim, blockdim](z_array, out)
     return out
 
 
@@ -707,7 +862,7 @@ def hotspots(raster, kernel):
         >>> import numpy as np
         >>> import xarray as xr
         >>> from xrspatial.convolution import custom_kernel
-        >>> kernel = custom_kernel(np.array([1, 1, 0]))
+        >>> kernel = custom_kernel(np.array([[1, 1, 0]]))
         >>> data = np.array([
         ...    [0, 1000, 1000, 0, 0, 0],
         ...    [0, 0, 0, -1000, -1000, 0],
@@ -722,8 +877,6 @@ def hotspots(raster, kernel):
         Dimensions without coordinates: dim_0, dim_1
     """
 
-    # TODO: edit unit of output raster to percent (%)
-
     # validate raster
     if not isinstance(raster, DataArray):
         raise TypeError("`raster` must be instance of DataArray")
@@ -740,7 +893,10 @@ def hotspots(raster, kernel):
     )
     out = mapper(raster)(raster, kernel)
 
+    attrs = copy.deepcopy(raster.attrs)
+    attrs['unit'] = '%'
+
     return DataArray(out,
                      coords=raster.coords,
                      dims=raster.dims,
-                     attrs=raster.attrs)
+                     attrs=attrs)
