@@ -7,7 +7,7 @@ import numpy as np
 import xarray
 
 from .gpu_rtx import has_rtx
-from .utils import has_cupy, is_cupy_array, ngjit
+from .utils import has_cuda_and_cupy, is_cupy_array, ngjit
 
 E_ROW_ID = 0
 E_COL_ID = 1
@@ -1637,7 +1637,7 @@ def viewshed(raster: xarray.DataArray,
         array([[ 0,  0,  1,  0,  0],
                [ 1,  3,  0,  0,  0],
                [10,  2,  5,  2, -1],
-               [20,  1,  2,  9,  0]])
+               [11,  1,  2,  9,  0]])
         Coordinates:
           * y        (y) float64 1.0 2.0 3.0 4.0
           * x        (x) float64 1.0 2.0 3.0 4.0 5.0
@@ -1656,7 +1656,7 @@ def viewshed(raster: xarray.DataArray,
     if isinstance(raster.data, np.ndarray):
         return _viewshed_cpu(raster, x, y, observer_elev, target_elev)
 
-    elif has_cupy() and is_cupy_array(raster.data):
+    elif has_cuda_and_cupy() and is_cupy_array(raster.data):
         if has_rtx():
             # Run on gpu
             from .gpu_rtx.viewshed import viewshed_gpu
